<?php

namespace App\Jobs;

use App\BO\BimFileBO;
use App\Libraries\BIM\BIMFactory;
use App\Models\BimFile;
use App\Models\File;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Http\UploadedFile;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;

/**
 * 处理BIM的轻量化转换
 */
class ConvertBimFile implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private $fileId;
    private BimFileBO $bimFileBO;
    private $file;

    /**
     * Create a new job instance.
     */
    public function __construct(int $fileId, BimFileBO $bimFileBO, ?UploadedFile $file = null)
    {
        $this->fileId = $fileId;
        $this->bimFileBO = $bimFileBO;
        $this->file = $file;
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {
        $file = File::find($this->fileId);

        if ($file) {
            $bimInfo = BIMFactory::make()->uploadFile($this->file, $this->bimFileBO);

            BimFile::query()->create([
                'file_id' => $file->id,
                ...$bimInfo
            ]);
        }
    }
}
